# SPDX-License-Identifier: (Apache-2.0 OR MIT)
# Copyright ijl (2022-2025), Aarni Koskela (2022)

import pytest

import orjson


@pytest.mark.parametrize(
    "val",
    [
        b'"\xc8\x93',
        b'"\xc8',
    ],
)
def test_invalid(val):
    with pytest.raises(orjson.JSONDecodeError):
        orjson.loads(val)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2026)

import lzma
import os
import sys
import sysconfig
from pathlib import Path
from typing import Any

IS_FREETHREADING = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))

SUPPORTS_MEMORYVIEW = not IS_FREETHREADING

SUPPORTS_BYTEARRAY = not IS_FREETHREADING

SUPPORTS_GETREFCOUNT = sys.implementation == "cpython"

numpy = None  # type: ignore
try:
    import numpy  # type: ignore # noqa: F401
except ImportError:
    pass

pandas = None  # type: ignore
try:
    import pandas  # type: ignore # noqa: F401
except ImportError:
    pass

import pytest

import orjson

data_dir = os.path.join(os.path.dirname(__file__), "../data")

STR_CACHE: dict[str, str] = {}

OBJ_CACHE: dict[str, Any] = {}


def read_fixture_bytes(filename, subdir=None):
    if subdir is None:
        path = Path(data_dir, filename)
    else:
        path = Path(data_dir, subdir, filename)
    if path.suffix == ".xz":
        contents = lzma.decompress(path.read_bytes())
    else:
        contents = path.read_bytes()
    return contents


def read_fixture_str(filename, subdir=None):
    if filename not in STR_CACHE:
        STR_CACHE[filename] = read_fixture_bytes(filename, subdir).decode("utf-8")
    return STR_CACHE[filename]


def read_fixture_obj(filename):
    if filename not in OBJ_CACHE:
        OBJ_CACHE[filename] = orjson.loads(read_fixture_str(filename))
    return OBJ_CACHE[filename]


needs_data = pytest.mark.skipif(
    not Path(data_dir).exists(),
    reason="Test depends on ./data dir that contains fixtures",
)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2025)

import orjson


def test_issue565():
    assert (
        orjson.dumps("\n\r\u000b\f\u001c\u001d\u001e")
        == b'"\\n\\r\\u000b\\f\\u001c\\u001d\\u001e"'
    )


def test_0x00():
    assert orjson.dumps("\u0000") == b'"\\u0000"'


def test_0x01():
    assert orjson.dumps("\u0001") == b'"\\u0001"'


def test_0x02():
    assert orjson.dumps("\u0002") == b'"\\u0002"'


def test_0x03():
    assert orjson.dumps("\u0003") == b'"\\u0003"'


def test_0x04():
    assert orjson.dumps("\u0004") == b'"\\u0004"'


def test_0x05():
    assert orjson.dumps("\u0005") == b'"\\u0005"'


def test_0x06():
    assert orjson.dumps("\u0006") == b'"\\u0006"'


def test_0x07():
    assert orjson.dumps("\u0007") == b'"\\u0007"'


def test_0x08():
    assert orjson.dumps("\u0008") == b'"\\b"'


def test_0x09():
    assert orjson.dumps("\u0009") == b'"\\t"'


def test_0x0a():
    assert orjson.dumps("\u000a") == b'"\\n"'


def test_0x0b():
    assert orjson.dumps("\u000b") == b'"\\u000b"'


def test_0x0c():
    assert orjson.dumps("\u000c") == b'"\\f"'


def test_0x0d():
    assert orjson.dumps("\u000d") == b'"\\r"'


def test_0x0e():
    assert orjson.dumps("\u000e") == b'"\\u000e"'


def test_0x0f():
    assert orjson.dumps("\u000f") == b'"\\u000f"'


def test_0x10():
    assert orjson.dumps("\u0010") == b'"\\u0010"'


def test_0x11():
    assert orjson.dumps("\u0011") == b'"\\u0011"'


def test_0x12():
    assert orjson.dumps("\u0012") == b'"\\u0012"'


def test_0x13():
    assert orjson.dumps("\u0013") == b'"\\u0013"'


def test_0x14():
    assert orjson.dumps("\u0014") == b'"\\u0014"'


def test_0x15():
    assert orjson.dumps("\u0015") == b'"\\u0015"'


def test_0x16():
    assert orjson.dumps("\u0016") == b'"\\u0016"'


def test_0x17():
    assert orjson.dumps("\u0017") == b'"\\u0017"'


def test_0x18():
    assert orjson.dumps("\u0018") == b'"\\u0018"'


def test_0x19():
    assert orjson.dumps("\u0019") == b'"\\u0019"'


def test_0x1a():
    assert orjson.dumps("\u001a") == b'"\\u001a"'


def test_backslash():
    assert orjson.dumps("\\") == b'"\\\\"'


def test_quote():
    assert orjson.dumps('"') == b'"\\""'



# SPDX-License-Identifier: (Apache-2.0 OR MIT)
# Copyright ijl (2019-2026), Aarni Koskela (2021)

import gc
from json import dumps as _json_dumps
from json import loads as json_loads

from orjson import dumps as orjson_dumps
from orjson import loads as orjson_loads

from .util import read_fixture


def json_dumps(obj):
    return _json_dumps(obj).encode("utf-8")


LIBRARIES = {
    "orjson": (orjson_dumps, orjson_loads),
    "json": (json_dumps, json_loads),
}


FIXTURE_NAMES = (
    "canada.json",
    "citm_catalog.json",
    "github.json",
    "twitter.json",
)

FIXTURE_AS_BYTES = {name: read_fixture(f"{name}.xz") for name in FIXTURE_NAMES}

FIXTURE_AS_OBJECTS = {
    name: orjson_loads(FIXTURE_AS_BYTES[name]) for name in FIXTURE_NAMES
}


if hasattr(gc, "freeze"):
    gc.freeze()
if hasattr(gc, "collect"):
    gc.collect()
if hasattr(gc, "disable"):
    gc.disable()



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2026)
# Test files from http://json.org/JSON_checker/

import pytest

import orjson

from .util import needs_data, read_fixture_str

PATTERN_1 = '["JSON Test Pattern pass1",{"object with 1 member":["array with 1 element"]},{},[],-42,true,false,null,{"integer":1234567890,"real":-9876.54321,"e":1.23456789e-13,"E":1.23456789e+34,"":2.3456789012e+76,"zero":0,"one":1,"space":" ","quote":"\\"","backslash":"\\\\","controls":"\\b\\f\\n\\r\\t","slash":"/ & /","alpha":"abcdefghijklmnopqrstuvwyz","ALPHA":"ABCDEFGHIJKLMNOPQRSTUVWYZ","digit":"0123456789","0123456789":"digit","special":"`1~!@#$%^&*()_+-={\':[,]}|;.</>?","hex":"ģ䕧覫췯ꯍ\uef4a","true":true,"false":false,"null":null,"array":[],"object":{},"address":"50 St. James Street","url":"http://www.JSON.org/","comment":"// /* <!-- --","# -- --> */":" "," s p a c e d ":[1,2,3,4,5,6,7],"compact":[1,2,3,4,5,6,7],"jsontext":"{\\"object with 1 member\\":[\\"array with 1 element\\"]}","quotes":"&#34; \\" %22 0x22 034 &#x22;","/\\\\\\"쫾몾ꮘﳞ볚\uef4a\\b\\f\\n\\r\\t`1~!@#$%^&*()_+-=[]{}|;:\',./<>?":"A key can be any string"},0.5,98.6,99.44,1066,10.0,1.0,0.1,1.0,2.0,2.0,"rosebud"]'.encode()


@needs_data
class TestJsonChecker:
    def _run_fail_json(self, filename, exc=orjson.JSONDecodeError):
        data = read_fixture_str(filename, "jsonchecker")
        pytest.raises(exc, orjson.loads, data)

    def _run_pass_json(self, filename, match=""):
        data = read_fixture_str(filename, "jsonchecker")
        assert orjson.dumps(orjson.loads(data)) == match

    def test_fail01(self):
        """
        fail01.json
        """
        self._run_pass_json(
            "fail01.json",
            b'"A JSON payload should be an object or array, not a string."',
        )

    def test_fail02(self):
        """
        fail02.json
        """
        self._run_fail_json("fail02.json", orjson.JSONDecodeError)  # EOF

    def test_fail03(self):
        """
        fail03.json
        """
        self._run_fail_json("fail03.json")

    def test_fail04(self):
        """
        fail04.json
        """
        self._run_fail_json("fail04.json")

    def test_fail05(self):
        """
        fail05.json
        """
        self._run_fail_json("fail05.json")

    def test_fail06(self):
        """
        fail06.json
        """
        self._run_fail_json("fail06.json")

    def test_fail07(self):
        """
        fail07.json
        """
        self._run_fail_json("fail07.json")

    def test_fail08(self):
        """
        fail08.json
        """
        self._run_fail_json("fail08.json")

    def test_fail09(self):
        """
        fail09.json
        """
        self._run_fail_json("fail09.json")

    def test_fail10(self):
        """
        fail10.json
        """
        self._run_fail_json("fail10.json")

    def test_fail11(self):
        """
        fail11.json
        """
        self._run_fail_json("fail11.json")

    def test_fail12(self):
        """
        fail12.json
        """
        self._run_fail_json("fail12.json")

    def test_fail13(self):
        """
        fail13.json
        """
        self._run_fail_json("fail13.json")

    def test_fail14(self):
        """
        fail14.json
        """
        self._run_fail_json("fail14.json")

    def test_fail15(self):
        """
        fail15.json
        """
        self._run_fail_json("fail15.json")

    def test_fail16(self):
        """
        fail16.json
        """
        self._run_fail_json("fail16.json")

    def test_fail17(self):
        """
        fail17.json
        """
        self._run_fail_json("fail17.json")

    def test_fail18(self):
        """
        fail18.json
        """
        self._run_pass_json(
            "fail18.json",
            b'[[[[[[[[[[[[[[[[[[[["Too deep"]]]]]]]]]]]]]]]]]]]]',
        )

    def test_fail19(self):
        """
        fail19.json
        """
        self._run_fail_json("fail19.json")

    def test_fail20(self):
        """
        fail20.json
        """
        self._run_fail_json("fail20.json")

    def test_fail21(self):
        """
        fail21.json
        """
        self._run_fail_json("fail21.json")

    def test_fail22(self):
        """
        fail22.json
        """
        self._run_fail_json("fail22.json")

    def test_fail23(self):
        """
        fail23.json
        """
        self._run_fail_json("fail23.json")

    def test_fail24(self):
        """
        fail24.json
        """
        self._run_fail_json("fail24.json")

    def test_fail25(self):
        """
        fail25.json
        """
        self._run_fail_json("fail25.json")

    def test_fail26(self):
        """
        fail26.json
        """
        self._run_fail_json("fail26.json")

    def test_fail27(self):
        """
        fail27.json
        """
        self._run_fail_json("fail27.json")

    def test_fail28(self):
        """
        fail28.json
        """
        self._run_fail_json("fail28.json")

    def test_fail29(self):
        """
        fail29.json
        """
        self._run_fail_json("fail29.json")

    def test_fail30(self):
        """
        fail30.json
        """
        self._run_fail_json("fail30.json")

    def test_fail31(self):
        """
        fail31.json
        """
        self._run_fail_json("fail31.json")

    def test_fail32(self):
        """
        fail32.json
        """
        self._run_fail_json("fail32.json", orjson.JSONDecodeError)  # EOF

    def test_fail33(self):
        """
        fail33.json
        """
        self._run_fail_json("fail33.json")

    def test_pass01(self):
        """
        pass01.json
        """
        self._run_pass_json("pass01.json", PATTERN_1)

    def test_pass02(self):
        """
        pass02.json
        """
        self._run_pass_json(
            "pass02.json",
            b'[[[[[[[[[[[[[[[[[[["Not too deep"]]]]]]]]]]]]]]]]]]]',
        )

    def test_pass03(self):
        """
        pass03.json
        """
        self._run_pass_json(
            "pass03.json",
            b'{"JSON Test Pattern pass3":{"The outermost value":"must be '
            b'an object or array.","In this test":"It is an object."}}',
        )



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2025)

import os

import pytest

import orjson

ORJSON_RUNNER_MEMORY_GIB = os.getenv("ORJSON_RUNNER_MEMORY_GIB", "")


@pytest.mark.skipif(
    not ORJSON_RUNNER_MEMORY_GIB,
    reason="ORJSON_RUNNER_MEMORY_GIB not defined",
)
def test_memory_loads():
    buffer_factor = 12
    segment = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    size = (
        (int(ORJSON_RUNNER_MEMORY_GIB) * 1024 * 1024 * 1024)
        // buffer_factor
        // len(segment)
    )
    doc = "".join(segment for _ in range(size))
    with pytest.raises(orjson.JSONDecodeError) as exc_info:
        _ = orjson.loads(doc)
    assert (
        str(exc_info.value)
        == "Not enough memory to allocate buffer for parsing: line 1 column 1 (char 0)"
    )



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2022)

import collections
import json

import pytest

import orjson


class SubStr(str):
    pass


class SubInt(int):
    pass


class SubDict(dict):
    pass


class SubList(list):
    pass


class SubFloat(float):
    pass


class SubTuple(tuple):
    pass


class TestSubclass:
    def test_subclass_str(self):
        assert orjson.dumps(SubStr("zxc")) == b'"zxc"'

    def test_subclass_str_invalid(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubStr("\ud800"))

    def test_subclass_int(self):
        assert orjson.dumps(SubInt(1)) == b"1"

    def test_subclass_int_64(self):
        for val in (9223372036854775807, -9223372036854775807):
            assert orjson.dumps(SubInt(val)) == str(val).encode("utf-8")

    def test_subclass_int_53(self):
        for val in (9007199254740992, -9007199254740992):
            with pytest.raises(orjson.JSONEncodeError):
                orjson.dumps(SubInt(val), option=orjson.OPT_STRICT_INTEGER)

    def test_subclass_dict(self):
        assert orjson.dumps(SubDict({"a": "b"})) == b'{"a":"b"}'

    def test_subclass_list(self):
        assert orjson.dumps(SubList(["a", "b"])) == b'["a","b"]'
        ref = [True] * 512
        assert orjson.loads(orjson.dumps(SubList(ref))) == ref

    def test_subclass_float(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubFloat(1.1))
        assert json.dumps(SubFloat(1.1)) == "1.1"

    def test_subclass_tuple(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubTuple((1, 2)))
        assert json.dumps(SubTuple((1, 2))) == "[1, 2]"

    def test_namedtuple(self):
        Point = collections.namedtuple("Point", ["x", "y"])
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(Point(1, 2))

    def test_subclass_circular_dict(self):
        obj = SubDict({})
        obj["obj"] = obj
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)

    def test_subclass_circular_list(self):
        obj = SubList([])
        obj.append(obj)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)

    def test_subclass_circular_nested(self):
        obj = SubDict({})
        obj["list"] = SubList([{"obj": obj}])
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)


class TestSubclassPassthrough:
    def test_subclass_str(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubStr("zxc"), option=orjson.OPT_PASSTHROUGH_SUBCLASS)

    def test_subclass_int(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubInt(1), option=orjson.OPT_PASSTHROUGH_SUBCLASS)

    def test_subclass_dict(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubDict({"a": "b"}), option=orjson.OPT_PASSTHROUGH_SUBCLASS)

    def test_subclass_list(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(SubList(["a", "b"]), option=orjson.OPT_PASSTHROUGH_SUBCLASS)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2020-2025)

import dataclasses
import datetime
import uuid

import pytest

import orjson

try:
    import pytz
except ImportError:
    pytz = None  # type: ignore

from .util import numpy


class SubStr(str):
    pass


class TestNonStrKeyTests:
    def test_dict_keys_duplicate(self):
        """
        OPT_NON_STR_KEYS serializes duplicate keys
        """
        assert (
            orjson.dumps({"1": True, 1: False}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"1":true,"1":false}'
        )

    def test_dict_keys_int(self):
        assert (
            orjson.dumps({1: True, 2: False}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"1":true,"2":false}'
        )

    def test_dict_keys_substr(self):
        assert (
            orjson.dumps({SubStr("aaa"): True}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"aaa":true}'
        )

    def test_dict_keys_substr_passthrough(self):
        """
        OPT_PASSTHROUGH_SUBCLASS does not affect OPT_NON_STR_KEYS
        """
        assert (
            orjson.dumps(
                {SubStr("aaa"): True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_PASSTHROUGH_SUBCLASS,
            )
            == b'{"aaa":true}'
        )

    def test_dict_keys_substr_invalid(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({SubStr("\ud800"): True}, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_strict(self):
        """
        OPT_NON_STR_KEYS does not respect OPT_STRICT_INTEGER
        """
        assert (
            orjson.dumps(
                {9223372036854775807: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"9223372036854775807":true}'
        )

    def test_dict_keys_int_range_valid_i64(self):
        """
        OPT_NON_STR_KEYS has a i64 range for int, valid
        """
        assert (
            orjson.dumps(
                {9223372036854775807: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"9223372036854775807":true}'
        )
        assert (
            orjson.dumps(
                {-9223372036854775807: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"-9223372036854775807":true}'
        )
        assert (
            orjson.dumps(
                {9223372036854775809: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"9223372036854775809":true}'
        )

    def test_dict_keys_int_range_valid_u64(self):
        """
        OPT_NON_STR_KEYS has a u64 range for int, valid
        """
        assert (
            orjson.dumps(
                {0: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"0":true}'
        )
        assert (
            orjson.dumps(
                {18446744073709551615: True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_STRICT_INTEGER,
            )
            == b'{"18446744073709551615":true}'
        )

    def test_dict_keys_int_range_invalid(self):
        """
        OPT_NON_STR_KEYS has a range of i64::MIN to u64::MAX
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({-9223372036854775809: True}, option=orjson.OPT_NON_STR_KEYS)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({18446744073709551616: True}, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_float(self):
        assert (
            orjson.dumps({1.1: True, 2.2: False}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"1.1":true,"2.2":false}'
        )

    def test_dict_keys_inf(self):
        assert (
            orjson.dumps({float("Infinity"): True}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"null":true}'
        )
        assert (
            orjson.dumps({float("-Infinity"): True}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"null":true}'
        )

    def test_dict_keys_nan(self):
        assert (
            orjson.dumps({float("NaN"): True}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"null":true}'
        )

    def test_dict_keys_bool(self):
        assert (
            orjson.dumps({True: True, False: False}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"true":true,"false":false}'
        )

    def test_dict_keys_datetime(self):
        assert (
            orjson.dumps(
                {datetime.datetime(2000, 1, 1, 2, 3, 4, 123): True},
                option=orjson.OPT_NON_STR_KEYS,
            )
            == b'{"2000-01-01T02:03:04.000123":true}'
        )

    def test_dict_keys_datetime_opt(self):
        assert (
            orjson.dumps(
                {datetime.datetime(2000, 1, 1, 2, 3, 4, 123): True},
                option=orjson.OPT_NON_STR_KEYS
                | orjson.OPT_OMIT_MICROSECONDS
                | orjson.OPT_NAIVE_UTC
                | orjson.OPT_UTC_Z,
            )
            == b'{"2000-01-01T02:03:04Z":true}'
        )

    def test_dict_keys_datetime_passthrough(self):
        """
        OPT_PASSTHROUGH_DATETIME does not affect OPT_NON_STR_KEYS
        """
        assert (
            orjson.dumps(
                {datetime.datetime(2000, 1, 1, 2, 3, 4, 123): True},
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_PASSTHROUGH_DATETIME,
            )
            == b'{"2000-01-01T02:03:04.000123":true}'
        )

    def test_dict_keys_uuid(self):
        """
        OPT_NON_STR_KEYS always serializes UUID as keys
        """
        assert (
            orjson.dumps(
                {uuid.UUID("7202d115-7ff3-4c81-a7c1-2a1f067b1ece"): True},
                option=orjson.OPT_NON_STR_KEYS,
            )
            == b'{"7202d115-7ff3-4c81-a7c1-2a1f067b1ece":true}'
        )

    def test_dict_keys_date(self):
        assert (
            orjson.dumps(
                {datetime.date(1970, 1, 1): True},
                option=orjson.OPT_NON_STR_KEYS,
            )
            == b'{"1970-01-01":true}'
        )

    def test_dict_keys_time(self):
        assert (
            orjson.dumps(
                {datetime.time(12, 15, 59, 111): True},
                option=orjson.OPT_NON_STR_KEYS,
            )
            == b'{"12:15:59.000111":true}'
        )

    def test_dict_non_str_and_sort_keys(self):
        assert (
            orjson.dumps(
                {
                    "other": 1,
                    datetime.date(1970, 1, 5): 2,
                    datetime.date(1970, 1, 3): 3,
                },
                option=orjson.OPT_NON_STR_KEYS | orjson.OPT_SORT_KEYS,
            )
            == b'{"1970-01-03":3,"1970-01-05":2,"other":1}'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_dict_keys_time_err(self):
        """
        OPT_NON_STR_KEYS propagates errors in types
        """
        val = datetime.time(12, 15, 59, 111, tzinfo=pytz.timezone("Asia/Shanghai"))
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({val: True}, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_str(self):
        assert (
            orjson.dumps({"1": True}, option=orjson.OPT_NON_STR_KEYS) == b'{"1":true}'
        )

    def test_dict_keys_type(self):
        class Obj:
            a: str

        val = Obj()
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({val: True}, option=orjson.OPT_NON_STR_KEYS)

    @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
    def test_dict_keys_array(self):
        with pytest.raises(TypeError):
            _ = {numpy.array([1, 2]): True}  # type: ignore

    def test_dict_keys_dataclass(self):
        @dataclasses.dataclass
        class Dataclass:
            a: str

        with pytest.raises(TypeError):
            _ = {Dataclass("a"): True}

    def test_dict_keys_dataclass_hash(self):
        @dataclasses.dataclass
        class Dataclass:
            a: str

            def __hash__(self):
                return 1

        obj = {Dataclass("a"): True}
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_list(self):
        with pytest.raises(TypeError):
            _ = {[]: True}

    def test_dict_keys_dict(self):
        with pytest.raises(TypeError):
            _ = {{}: True}

    def test_dict_keys_tuple(self):
        obj = {(): True}
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_unknown(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({frozenset(): True}, option=orjson.OPT_NON_STR_KEYS)

    def test_dict_keys_no_str_call(self):
        class Obj:
            a: str

            def __str__(self):
                return "Obj"

        val = Obj()
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({val: True}, option=orjson.OPT_NON_STR_KEYS)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2026)


import pytest

import orjson


def make_recursive_list_dict(limit: int, envelope_key: str, recurse_key: str):
    i = 0
    root = [{envelope_key: i, recurse_key: []}]
    i += 1
    while i < limit:
        sub = [{envelope_key: i, recurse_key: []}]
        sub[0][recurse_key] = root
        root = sub
        i += 1
    return root


class TestSerializeRecursion:
    @pytest.mark.parametrize("i", range(1, 127))
    def test_dumps_recursion_valid_long(self, i):
        root = make_recursive_list_dict(i, "🐈" * 512, "b" * 1024)
        orjson.dumps(root)

    @pytest.mark.parametrize("i", range(1, 127))
    def test_dumps_recursion_valid_short_1(self, i):
        root = make_recursive_list_dict(i, "a", "")
        orjson.dumps(root)

    @pytest.mark.parametrize("i", range(1, 127))
    def test_dumps_recursion_valid_short_2(self, i):
        root = make_recursive_list_dict(i, "level", "next")
        orjson.dumps(root)

    def test_dumps_recursion_limit(self):
        root = make_recursive_list_dict(128, "level", "next")
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(root)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2020-2025)

import datetime
import enum

import pytest

import orjson


class StrEnum(str, enum.Enum):
    AAA = "aaa"


class IntEnum(int, enum.Enum):
    ONE = 1


class IntEnumEnum(enum.IntEnum):
    ONE = 1


class IntFlagEnum(enum.IntFlag):
    ONE = 1


class FlagEnum(enum.Flag):
    ONE = 1


class AutoEnum(enum.auto):
    A = "a"


class FloatEnum(float, enum.Enum):
    ONE = 1.1


class Custom:
    def __init__(self, val):
        self.val = val


def default(obj):
    if isinstance(obj, Custom):
        return obj.val
    raise TypeError


class UnspecifiedEnum(enum.Enum):
    A = "a"
    B = 1
    C = FloatEnum.ONE
    D = {"d": IntEnum.ONE}  # noqa: RUF012
    E = Custom("c")
    F = datetime.datetime(1970, 1, 1)


class TestEnum:
    def test_cannot_subclass(self):
        """
        enum.Enum cannot be subclassed

        obj->ob_type->ob_base will always be enum.EnumMeta
        """
        with pytest.raises(TypeError):

            class Subclass(StrEnum):  # type: ignore
                B = "b"

    def test_arbitrary_enum(self):
        assert orjson.dumps(UnspecifiedEnum.A) == b'"a"'
        assert orjson.dumps(UnspecifiedEnum.B) == b"1"
        assert orjson.dumps(UnspecifiedEnum.C) == b"1.1"
        assert orjson.dumps(UnspecifiedEnum.D) == b'{"d":1}'

    def test_custom_enum(self):
        assert orjson.dumps(UnspecifiedEnum.E, default=default) == b'"c"'

    def test_enum_options(self):
        assert (
            orjson.dumps(UnspecifiedEnum.F, option=orjson.OPT_NAIVE_UTC)
            == b'"1970-01-01T00:00:00+00:00"'
        )

    def test_int_enum(self):
        assert orjson.dumps(IntEnum.ONE) == b"1"

    def test_intenum_enum(self):
        assert orjson.dumps(IntEnumEnum.ONE) == b"1"

    def test_intflag_enum(self):
        assert orjson.dumps(IntFlagEnum.ONE) == b"1"

    def test_flag_enum(self):
        assert orjson.dumps(FlagEnum.ONE) == b"1"

    def test_auto_enum(self):
        assert orjson.dumps(AutoEnum.A) == b'"a"'

    def test_float_enum(self):
        assert orjson.dumps(FloatEnum.ONE) == b"1.1"

    def test_str_enum(self):
        assert orjson.dumps(StrEnum.AAA) == b'"aaa"'

    def test_bool_enum(self):
        with pytest.raises(TypeError):

            class BoolEnum(bool, enum.Enum):  # type: ignore
                TRUE = True

    def test_non_str_keys_enum(self):
        assert (
            orjson.dumps({StrEnum.AAA: 1}, option=orjson.OPT_NON_STR_KEYS)
            == b'{"aaa":1}'
        )
        assert (
            orjson.dumps({IntEnum.ONE: 1}, option=orjson.OPT_NON_STR_KEYS) == b'{"1":1}'
        )



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2019-2023)

import orjson

try:
    from typing import TypedDict  # type: ignore
except ImportError:
    from typing_extensions import TypedDict


class TestTypedDict:
    def test_typeddict(self):
        """
        dumps() TypedDict
        """

        class TypedDict1(TypedDict):
            a: str
            b: int

        obj = TypedDict1(a="a", b=1)
        assert orjson.dumps(obj) == b'{"a":"a","b":1}'



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2025)

import pytest

import orjson

from .util import needs_data, read_fixture_bytes, read_fixture_str


@needs_data
class TestFixture:
    def test_twitter(self):
        """
        loads(),dumps() twitter.json
        """
        val = read_fixture_str("twitter.json.xz")
        read = orjson.loads(val)
        assert orjson.loads(orjson.dumps(read)) == read

    @needs_data
    def test_canada(self):
        """
        loads(), dumps() canada.json
        """
        val = read_fixture_str("canada.json.xz")
        read = orjson.loads(val)
        assert orjson.loads(orjson.dumps(read)) == read

    def test_citm_catalog(self):
        """
        loads(), dumps() citm_catalog.json
        """
        val = read_fixture_str("citm_catalog.json.xz")
        read = orjson.loads(val)
        assert orjson.loads(orjson.dumps(read)) == read

    def test_github(self):
        """
        loads(), dumps() github.json
        """
        val = read_fixture_str("github.json.xz")
        read = orjson.loads(val)
        assert orjson.loads(orjson.dumps(read)) == read

    def test_blns(self):
        """
        loads() blns.json JSONDecodeError

        https://github.com/minimaxir/big-list-of-naughty-strings
        """
        val = read_fixture_bytes("blns.txt.xz")
        for line in val.split(b"\n"):
            if line and not line.startswith(b"#"):
                with pytest.raises(orjson.JSONDecodeError):
                    _ = orjson.loads(b'"' + val + b'"')



# SPDX-License-Identifier: (Apache-2.0 OR MIT)
# Copyright ijl (2018-2026), hauntsaninja (2020)

import datetime
import inspect
import json
import re

import pytest

import orjson

SIMPLE_TYPES = (1, 1.0, -1, None, "str", True, False)

LOADS_RECURSION_LIMIT = 1024


def default(obj):
    return str(obj)


class TestApi:
    def test_loads_trailing(self):
        """
        loads() handles trailing whitespace
        """
        assert orjson.loads("{}\n\t ") == {}

    def test_loads_trailing_invalid(self):
        """
        loads() handles trailing invalid
        """
        pytest.raises(orjson.JSONDecodeError, orjson.loads, "{}\n\t a")

    def test_simple_json(self):
        """
        dumps() equivalent to json on simple types
        """
        for obj in SIMPLE_TYPES:
            assert orjson.dumps(obj) == json.dumps(obj).encode("utf-8")

    def test_simple_round_trip(self):
        """
        dumps(), loads() round trip on simple types
        """
        for obj in SIMPLE_TYPES:
            assert orjson.loads(orjson.dumps(obj)) == obj

    def test_loads_type(self):
        """
        loads() invalid type
        """
        for val in (1, 3.14, [], {}, None):  # type: ignore
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)  # type: ignore

    def test_loads_recursion_partial(self):
        """
        loads() recursion limit partial
        """
        pytest.raises(orjson.JSONDecodeError, orjson.loads, "[" * (1024 * 1024))

    def test_loads_recursion_valid_limit_array(self):
        """
        loads() recursion limit at limit array
        """
        n = LOADS_RECURSION_LIMIT + 1
        value = b"[" * n + b"]" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_limit_object(self):
        """
        loads() recursion limit at limit object
        """
        n = LOADS_RECURSION_LIMIT
        value = b'{"key":' * n + b'{"key":true}' + b"}" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_limit_mixed(self):
        """
        loads() recursion limit at limit mixed
        """
        n = LOADS_RECURSION_LIMIT
        value = b"".join((b"[", b'{"key":' * n, b'{"key":true}' + b"}" * n, b"]"))
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_excessive_array(self):
        """
        loads() recursion limit excessively high value
        """
        n = 10000000
        value = b"[" * n + b"]" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_limit_array_pretty(self):
        """
        loads() recursion limit at limit array pretty
        """
        n = LOADS_RECURSION_LIMIT + 1
        value = b"[\n  " * n + b"]" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_limit_object_pretty(self):
        """
        loads() recursion limit at limit object pretty
        """
        n = LOADS_RECURSION_LIMIT
        value = b'{\n  "key":' * n + b'{"key":true}' + b"}" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_limit_mixed_pretty(self):
        """
        loads() recursion limit at limit mixed pretty
        """
        n = LOADS_RECURSION_LIMIT
        value = b'[\n  {"key":' * n + b'{"key":true}' + b"}" * n + b"]"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_loads_recursion_valid_excessive_array_pretty(self):
        """
        loads() recursion limit excessively high value pretty
        """
        n = 10000000
        value = b"[\n  " * n + b"]" * n
        pytest.raises(orjson.JSONDecodeError, orjson.loads, value)

    def test_version(self):
        """
        __version__
        """
        assert re.match(r"^\d+\.\d+(\.\d+)?$", orjson.__version__)

    def test_valueerror(self):
        """
        orjson.JSONDecodeError is a subclass of ValueError
        """
        pytest.raises(orjson.JSONDecodeError, orjson.loads, "{")
        pytest.raises(ValueError, orjson.loads, "{")

    def test_optional_none(self):
        """
        dumps() option, default None
        """
        assert orjson.dumps([], option=None) == b"[]"
        assert orjson.dumps([], default=None) == b"[]"
        assert orjson.dumps([], option=None, default=None) == b"[]"
        assert orjson.dumps([], None, None) == b"[]"

    def test_option_not_int(self):
        """
        dumps() option not int or None
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(True, option=True)

    def test_option_invalid_int(self):
        """
        dumps() option invalid 64-bit number
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(True, option=9223372036854775809)

    def test_option_range_low(self):
        """
        dumps() option out of range low
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(True, option=-1)

    def test_option_range_high(self):
        """
        dumps() option out of range high
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(True, option=1 << 12)

    def test_opts_multiple(self):
        """
        dumps() multiple option
        """
        assert (
            orjson.dumps(
                [1, datetime.datetime(2000, 1, 1, 2, 3, 4)],
                option=orjson.OPT_STRICT_INTEGER | orjson.OPT_NAIVE_UTC,
            )
            == b'[1,"2000-01-01T02:03:04+00:00"]'
        )

    def test_default_positional(self):
        """
        dumps() positional arg
        """
        with pytest.raises(TypeError):
            orjson.dumps(__obj={})  # type: ignore
        with pytest.raises(TypeError):
            orjson.dumps(zxc={})  # type: ignore

    def test_default_unknown_kwarg(self):
        """
        dumps() unknown kwarg
        """
        with pytest.raises(TypeError):
            orjson.dumps({}, zxc=default)  # type: ignore

    def test_default_empty_kwarg(self):
        """
        dumps() empty kwarg
        """
        assert orjson.dumps(None) == b"null"

    def test_default_twice(self):
        """
        dumps() default twice
        """
        with pytest.raises(TypeError):
            orjson.dumps({}, default, default=default)  # type: ignore

    def test_option_twice(self):
        """
        dumps() option twice
        """
        with pytest.raises(TypeError):
            orjson.dumps({}, None, orjson.OPT_NAIVE_UTC, option=orjson.OPT_NAIVE_UTC)  # type: ignore

    def test_option_mixed(self):
        """
        dumps() option one arg, one kwarg
        """

        class Custom:
            def __str__(self):
                return "zxc"

        assert (
            orjson.dumps(
                [Custom(), datetime.datetime(2000, 1, 1, 2, 3, 4)],
                default,
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["zxc","2000-01-01T02:03:04+00:00"]'
        )

    def test_dumps_signature(self):
        """
        dumps() valid __text_signature__
        """
        assert (
            str(inspect.signature(orjson.dumps))
            == "(obj, /, default=None, option=None)"
        )
        inspect.signature(orjson.dumps).bind("str")
        inspect.signature(orjson.dumps).bind("str", default=default, option=1)
        inspect.signature(orjson.dumps).bind("str", default=None, option=None)

    def test_loads_signature(self):
        """
        loads() valid __text_signature__
        """
        assert str(inspect.signature(orjson.loads)), "(obj == /)"
        inspect.signature(orjson.loads).bind("[]")

    def test_dumps_module_str(self):
        """
        orjson.dumps.__module__ is a str
        """
        assert orjson.dumps.__module__ == "orjson"

    def test_loads_module_str(self):
        """
        orjson.loads.__module__ is a str
        """
        assert orjson.loads.__module__ == "orjson"

    def test_bytes_buffer(self):
        """
        dumps() trigger buffer growing where length is greater than growth
        """
        a = "a" * 900
        b = "b" * 4096
        c = "c" * 4096 * 4096
        assert orjson.dumps([a, b, c]) == f'["{a}","{b}","{c}"]'.encode("utf-8")

    def test_bytes_null_terminated(self):
        """
        dumps() PyBytesObject buffer is null-terminated
        """
        # would raise ValueError: invalid literal for int() with base 10: b'1596728892'
        int(orjson.dumps(1596728892))



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2026)

import io
import sys

import pytest

import orjson

from .util import SUPPORTS_BYTEARRAY, SUPPORTS_MEMORYVIEW


class TestType:
    def test_fragment(self):
        """
        orjson.JSONDecodeError on fragments
        """
        for val in ("n", "{", "[", "t"):
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_invalid(self):
        """
        orjson.JSONDecodeError on invalid
        """
        for val in ('{"age", 44}', "[31337,]", "[,31337]", "[]]", "[,]"):
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_str(self):
        """
        str
        """
        for obj, ref in (("blah", b'"blah"'), ("東京", b'"\xe6\x9d\xb1\xe4\xba\xac"')):
            assert orjson.dumps(obj) == ref
            assert orjson.loads(ref) == obj

    def test_str_latin1(self):
        """
        str latin1
        """
        assert orjson.loads(orjson.dumps("üýþÿ")) == "üýþÿ"

    def test_str_long(self):
        """
        str long
        """
        for obj in ("aaaa" * 1024, "üýþÿ" * 1024, "好" * 1024, "�" * 1024):
            assert orjson.loads(orjson.dumps(obj)) == obj

    def test_str_2mib(self):
        ref = '🐈🐈🐈🐈🐈"üýa0s9999🐈🐈🐈🐈🐈9\0999\\9999' * 1024 * 50
        assert orjson.loads(orjson.dumps(ref)) == ref

    def test_str_very_long(self):
        """
        str long enough to trigger overflow in bytecount
        """
        for obj in ("aaaa" * 20000, "üýþÿ" * 20000, "好" * 20000, "�" * 20000):
            assert orjson.loads(orjson.dumps(obj)) == obj

    def test_str_replacement(self):
        """
        str roundtrip �
        """
        assert orjson.dumps("�") == b'"\xef\xbf\xbd"'
        assert orjson.loads(b'"\xef\xbf\xbd"') == "�"

    def test_str_trailing_4_byte(self):
        ref = "うぞ〜😏🙌"
        assert orjson.loads(orjson.dumps(ref)) == ref

    def test_str_ascii_control(self):
        """
        worst case format_escaped_str_with_escapes() allocation
        """
        ref = "\x01\x1f" * 1024 * 16
        assert orjson.loads(orjson.dumps(ref)) == ref
        assert orjson.loads(orjson.dumps(ref, option=orjson.OPT_INDENT_2)) == ref

    def test_str_escape_quote_0(self):
        assert orjson.dumps('"aaaaaaabb') == b'"\\"aaaaaaabb"'

    def test_str_escape_quote_1(self):
        assert orjson.dumps('a"aaaaaabb') == b'"a\\"aaaaaabb"'

    def test_str_escape_quote_2(self):
        assert orjson.dumps('aa"aaaaabb') == b'"aa\\"aaaaabb"'

    def test_str_escape_quote_3(self):
        assert orjson.dumps('aaa"aaaabb') == b'"aaa\\"aaaabb"'

    def test_str_escape_quote_4(self):
        assert orjson.dumps('aaaa"aaabb') == b'"aaaa\\"aaabb"'

    def test_str_escape_quote_5(self):
        assert orjson.dumps('aaaaa"aabb') == b'"aaaaa\\"aabb"'

    def test_str_escape_quote_6(self):
        assert orjson.dumps('aaaaaa"abb') == b'"aaaaaa\\"abb"'

    def test_str_escape_quote_7(self):
        assert orjson.dumps('aaaaaaa"bb') == b'"aaaaaaa\\"bb"'

    def test_str_escape_quote_8(self):
        assert orjson.dumps('aaaaaaaab"') == b'"aaaaaaaab\\""'

    def test_str_escape_quote_multi(self):
        assert (
            orjson.dumps('aa"aaaaabbbbbbbbbbbbbbbbbbbb"bb')
            == b'"aa\\"aaaaabbbbbbbbbbbbbbbbbbbb\\"bb"'
        )

    def test_str_escape_quote_buffer(self):
        orjson.dumps(['"' * 4096] * 1024)

    def test_str_escape_backslash_0(self):
        assert orjson.dumps("\\aaaaaaabb") == b'"\\\\aaaaaaabb"'

    def test_str_escape_backslash_1(self):
        assert orjson.dumps("a\\aaaaaabb") == b'"a\\\\aaaaaabb"'

    def test_str_escape_backslash_2(self):
        assert orjson.dumps("aa\\aaaaabb") == b'"aa\\\\aaaaabb"'

    def test_str_escape_backslash_3(self):
        assert orjson.dumps("aaa\\aaaabb") == b'"aaa\\\\aaaabb"'

    def test_str_escape_backslash_4(self):
        assert orjson.dumps("aaaa\\aaabb") == b'"aaaa\\\\aaabb"'

    def test_str_escape_backslash_5(self):
        assert orjson.dumps("aaaaa\\aabb") == b'"aaaaa\\\\aabb"'

    def test_str_escape_backslash_6(self):
        assert orjson.dumps("aaaaaa\\abb") == b'"aaaaaa\\\\abb"'

    def test_str_escape_backslash_7(self):
        assert orjson.dumps("aaaaaaa\\bb") == b'"aaaaaaa\\\\bb"'

    def test_str_escape_backslash_8(self):
        assert orjson.dumps("aaaaaaaab\\") == b'"aaaaaaaab\\\\"'

    def test_str_escape_backslash_multi(self):
        assert (
            orjson.dumps("aa\\aaaaabbbbbbbbbbbbbbbbbbbb\\bb")
            == b'"aa\\\\aaaaabbbbbbbbbbbbbbbbbbbb\\\\bb"'
        )

    def test_str_escape_backslash_buffer(self):
        orjson.dumps(["\\" * 4096] * 1024)

    def test_str_escape_x32_0(self):
        assert orjson.dumps("\taaaaaaabb") == b'"\\taaaaaaabb"'

    def test_str_escape_x32_1(self):
        assert orjson.dumps("a\taaaaaabb") == b'"a\\taaaaaabb"'

    def test_str_escape_x32_2(self):
        assert orjson.dumps("aa\taaaaabb") == b'"aa\\taaaaabb"'

    def test_str_escape_x32_3(self):
        assert orjson.dumps("aaa\taaaabb") == b'"aaa\\taaaabb"'

    def test_str_escape_x32_4(self):
        assert orjson.dumps("aaaa\taaabb") == b'"aaaa\\taaabb"'

    def test_str_escape_x32_5(self):
        assert orjson.dumps("aaaaa\taabb") == b'"aaaaa\\taabb"'

    def test_str_escape_x32_6(self):
        assert orjson.dumps("aaaaaa\tabb") == b'"aaaaaa\\tabb"'

    def test_str_escape_x32_7(self):
        assert orjson.dumps("aaaaaaa\tbb") == b'"aaaaaaa\\tbb"'

    def test_str_escape_x32_8(self):
        assert orjson.dumps("aaaaaaaab\t") == b'"aaaaaaaab\\t"'

    def test_str_escape_x32_multi(self):
        assert (
            orjson.dumps("aa\taaaaabbbbbbbbbbbbbbbbbbbb\tbb")
            == b'"aa\\taaaaabbbbbbbbbbbbbbbbbbbb\\tbb"'
        )

    def test_str_escape_x32_buffer(self):
        orjson.dumps(["\t" * 4096] * 1024)

    def test_str_emoji(self):
        ref = "®️"
        assert orjson.loads(orjson.dumps(ref)) == ref

    def test_str_emoji_escape(self):
        ref = '/"®️/"'
        assert orjson.loads(orjson.dumps(ref)) == ref

    def test_very_long_list(self):
        orjson.dumps([[]] * 1024 * 16)

    def test_very_long_list_pretty(self):
        orjson.dumps([[]] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_dict(self):
        orjson.dumps([{}] * 1024 * 16)

    def test_very_long_dict_pretty(self):
        orjson.dumps([{}] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_str_empty(self):
        orjson.dumps([""] * 1024 * 16)

    def test_very_long_str_empty_pretty(self):
        orjson.dumps([""] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_str_not_empty(self):
        orjson.dumps(["a"] * 1024 * 16)

    def test_very_long_str_not_empty_pretty(self):
        orjson.dumps(["a"] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_bool(self):
        orjson.dumps([True] * 1024 * 16)

    def test_very_long_bool_pretty(self):
        orjson.dumps([True] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_int(self):
        orjson.dumps([(2**64) - 1] * 1024 * 16)

    def test_very_long_int_pretty(self):
        orjson.dumps([(2**64) - 1] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_very_long_float(self):
        orjson.dumps([sys.float_info.max] * 1024 * 16)

    def test_very_long_float_pretty(self):
        orjson.dumps([sys.float_info.max] * 1024 * 16, option=orjson.OPT_INDENT_2)

    def test_str_surrogates_loads(self):
        """
        str unicode surrogates loads()
        """
        pytest.raises(orjson.JSONDecodeError, orjson.loads, '"\ud800"')
        pytest.raises(orjson.JSONDecodeError, orjson.loads, '"\ud83d\ude80"')
        pytest.raises(orjson.JSONDecodeError, orjson.loads, '"\udcff"')
        pytest.raises(
            orjson.JSONDecodeError,
            orjson.loads,
            b'"\xed\xa0\xbd\xed\xba\x80"',
        )  # \ud83d\ude80

    def test_str_surrogates_dumps(self):
        """
        str unicode surrogates dumps()
        """
        pytest.raises(orjson.JSONEncodeError, orjson.dumps, "\ud800")
        pytest.raises(orjson.JSONEncodeError, orjson.dumps, "\ud83d\ude80")
        pytest.raises(orjson.JSONEncodeError, orjson.dumps, "\udcff")
        pytest.raises(orjson.JSONEncodeError, orjson.dumps, {"\ud83d\ude80": None})
        pytest.raises(
            orjson.JSONEncodeError,
            orjson.dumps,
            b"\xed\xa0\xbd\xed\xba\x80",
        )  # \ud83d\ude80

    def test_bytes_dumps(self):
        """
        bytes dumps not supported
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps([b"a"])

    def test_bytes_loads(self):
        """
        bytes loads
        """
        assert orjson.loads(b"[]") == []

    @pytest.mark.skipif(SUPPORTS_BYTEARRAY is False, reason="bytearray")
    def test_bytearray_loads(self):
        """
        bytearray loads
        """
        arr = bytearray()
        arr.extend(b"[]")
        assert orjson.loads(arr) == []

    @pytest.mark.skipif(SUPPORTS_MEMORYVIEW is False, reason="memoryview")
    def test_memoryview_loads_supported(self):
        """
        memoryview loads supported
        """
        assert orjson.loads(memoryview(b"[]")) == []

    @pytest.mark.skipif(SUPPORTS_MEMORYVIEW is True, reason="memoryview")
    def test_memoryview_loads_unsupported(self):
        """
        memoryview loads unsupported
        """
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads(memoryview(b"[]"))

    @pytest.mark.skipif(SUPPORTS_BYTEARRAY is False, reason="bytearray")
    def test_bytesio_loads_supported(self):
        """
        BytesIO loads supported
        """
        arr = io.BytesIO(b"[]")
        assert orjson.loads(arr.getbuffer()) == []

    @pytest.mark.skipif(SUPPORTS_BYTEARRAY is True, reason="bytearray")
    def test_bytesio_loads_unsupported(self):
        """
        BytesIO loads unsupported
        """
        arr = io.BytesIO(b"[]")
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads(arr.getbuffer())

    def test_bool(self):
        """
        bool
        """
        for obj, ref in ((True, "true"), (False, "false")):
            assert orjson.dumps(obj) == ref.encode("utf-8")
            assert orjson.loads(ref) == obj

    def test_bool_true_array(self):
        """
        bool true array
        """
        obj = [True] * 256
        ref = ("[" + ("true," * 255) + "true]").encode("utf-8")
        assert orjson.dumps(obj) == ref
        assert orjson.loads(ref) == obj

    def test_bool_false_array(self):
        """
        bool false array
        """
        obj = [False] * 256
        ref = ("[" + ("false," * 255) + "false]").encode("utf-8")
        assert orjson.dumps(obj) == ref
        assert orjson.loads(ref) == obj

    def test_none(self):
        """
        null
        """
        obj = None
        ref = "null"
        assert orjson.dumps(obj) == ref.encode("utf-8")
        assert orjson.loads(ref) == obj

    def test_int(self):
        """
        int compact and non-compact
        """
        obj = [-5000, -1000, -10, -5, -2, -1, 0, 1, 2, 5, 10, 1000, 50000]
        ref = b"[-5000,-1000,-10,-5,-2,-1,0,1,2,5,10,1000,50000]"
        assert orjson.dumps(obj) == ref
        assert orjson.loads(ref) == obj

    def test_null_array(self):
        """
        null array
        """
        obj = [None] * 256
        ref = ("[" + ("null," * 255) + "null]").encode("utf-8")
        assert orjson.dumps(obj) == ref
        assert orjson.loads(ref) == obj

    def test_nan_dumps(self):
        """
        NaN serializes to null
        """
        assert orjson.dumps(float("NaN")) == b"null"

    def test_nan_loads(self):
        """
        NaN is not valid JSON
        """
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[NaN]")
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[nan]")

    def test_infinity_dumps(self):
        """
        Infinity serializes to null
        """
        assert orjson.dumps(float("Infinity")) == b"null"

    def test_infinity_loads(self):
        """
        Infinity, -Infinity is not valid JSON
        """
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[infinity]")
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[Infinity]")
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[-Infinity]")
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads("[-infinity]")

    def test_int_53(self):
        """
        int 53-bit
        """
        for val in (9007199254740991, -9007199254740991):
            assert orjson.loads(str(val)) == val
            assert orjson.dumps(val, option=orjson.OPT_STRICT_INTEGER) == str(
                val,
            ).encode("utf-8")

    def test_int_53_exc(self):
        """
        int 53-bit exception on 64-bit
        """
        for val in (9007199254740992, -9007199254740992):
            with pytest.raises(orjson.JSONEncodeError):
                orjson.dumps(val, option=orjson.OPT_STRICT_INTEGER)

    def test_int_53_exc_usize(self):
        """
        int 53-bit exception on 64-bit usize
        """
        for val in (9223372036854775808, 18446744073709551615):
            with pytest.raises(orjson.JSONEncodeError):
                orjson.dumps(val, option=orjson.OPT_STRICT_INTEGER)

    def test_int_53_exc_128(self):
        """
        int 53-bit exception on 128-bit
        """
        val = 2**65
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(val, option=orjson.OPT_STRICT_INTEGER)

    def test_int_64(self):
        """
        int 64-bit
        """
        for val in (9223372036854775807, -9223372036854775807):
            assert orjson.loads(str(val)) == val
            assert orjson.dumps(val) == str(val).encode("utf-8")

    def test_uint_64(self):
        """
        uint 64-bit
        """
        for val in (0, 9223372036854775808, 18446744073709551615):
            assert orjson.loads(str(val)) == val
            assert orjson.dumps(val) == str(val).encode("utf-8")

    def test_int_128(self):
        """
        int 128-bit
        """
        for val in (18446744073709551616, -9223372036854775809):
            pytest.raises(orjson.JSONEncodeError, orjson.dumps, val)

    def test_float(self):
        """
        float
        """
        assert -1.1234567893 == orjson.loads("-1.1234567893")
        assert -1.234567893 == orjson.loads("-1.234567893")
        assert -1.34567893 == orjson.loads("-1.34567893")
        assert -1.4567893 == orjson.loads("-1.4567893")
        assert -1.567893 == orjson.loads("-1.567893")
        assert -1.67893 == orjson.loads("-1.67893")
        assert -1.7893 == orjson.loads("-1.7893")
        assert -1.893 == orjson.loads("-1.893")
        assert -1.3 == orjson.loads("-1.3")

        assert 1.1234567893 == orjson.loads("1.1234567893")
        assert 1.234567893 == orjson.loads("1.234567893")
        assert 1.34567893 == orjson.loads("1.34567893")
        assert 1.4567893 == orjson.loads("1.4567893")
        assert 1.567893 == orjson.loads("1.567893")
        assert 1.67893 == orjson.loads("1.67893")
        assert 1.7893 == orjson.loads("1.7893")
        assert 1.893 == orjson.loads("1.893")
        assert 1.3 == orjson.loads("1.3")

    def test_float_precision_loads(self):
        """
        float precision loads()
        """
        assert orjson.loads("31.245270191439438") == 31.245270191439438
        assert orjson.loads("-31.245270191439438") == -31.245270191439438
        assert orjson.loads("121.48791951161945") == 121.48791951161945
        assert orjson.loads("-121.48791951161945") == -121.48791951161945
        assert orjson.loads("100.78399658203125") == 100.78399658203125
        assert orjson.loads("-100.78399658203125") == -100.78399658203125

    def test_float_precision_dumps(self):
        """
        float precision dumps()
        """
        assert orjson.dumps(31.245270191439438) == b"31.245270191439438"
        assert orjson.dumps(-31.245270191439438) == b"-31.245270191439438"
        assert orjson.dumps(121.48791951161945) == b"121.48791951161945"
        assert orjson.dumps(-121.48791951161945) == b"-121.48791951161945"
        assert orjson.dumps(100.78399658203125) == b"100.78399658203125"
        assert orjson.dumps(-100.78399658203125) == b"-100.78399658203125"

    def test_float_edge(self):
        """
        float edge cases
        """
        assert orjson.dumps(0.8701) == b"0.8701"

        assert orjson.loads("0.8701") == 0.8701
        assert (
            orjson.loads("0.0000000000000000000000000000000000000000000000000123e50")
            == 1.23
        )
        assert orjson.loads("0.4e5") == 40000.0
        assert orjson.loads("0.00e-00") == 0.0
        assert orjson.loads("0.4e-001") == 0.04
        assert orjson.loads("0.123456789e-12") == 1.23456789e-13
        assert orjson.loads("1.234567890E+34") == 1.23456789e34
        assert orjson.loads("23456789012E66") == 2.3456789012e76

    def test_float_notation(self):
        """
        float notation
        """
        for val in ("1.337E40", "1.337e+40", "1337e40", "1.337E-4"):
            obj = orjson.loads(val)
            assert obj == float(val)
            assert orjson.dumps(val) == (f'"{val}"').encode("utf-8")

    def test_list(self):
        """
        list
        """
        obj = ["a", "😊", True, {"b": 1.1}, 2]
        ref = '["a","😊",true,{"b":1.1},2]'
        assert orjson.dumps(obj) == ref.encode("utf-8")
        assert orjson.loads(ref) == obj

    def test_tuple(self):
        """
        tuple
        """
        obj = ("a", "😊", True, {"b": 1.1}, 2)
        ref = '["a","😊",true,{"b":1.1},2]'
        assert orjson.dumps(obj) == ref.encode("utf-8")
        assert orjson.loads(ref) == list(obj)

    def test_object(self):
        """
        object() dumps()
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(object())



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2023-2025)

import datetime

import orjson

from .util import needs_data, read_fixture_bytes

FIXTURE_ISSUE_335 = {
    "pfkrpavmb": "maxyjzmvacdwjfiifmzwbztjmnqdsjesykpf",
    "obtsdcnmi": "psyucdnwjr",
    "ghsccsccdwep": 1673954411550,
    "vyqvkq": "ilfcrjas",
    "drfobem": {
        "mzqwuvwsglxx": 1673954411550,
        "oup": "mmimyli",
        "pxfepg": {
            "pnqjr": "ylttscz",
            "rahfmy": "xrcsutu",
            "rccgrkom": "fbt",
            "xulnoryigkhtoybq": "hubxdjrnaq",
            "vdwriwvlgu": datetime.datetime(
                2023,
                1,
                15,
                15,
                23,
                38,
                686000,
                tzinfo=datetime.timezone.utc,
            ),
            "fhmjsszqmxwfruiq": "fzghfrbjxqccf",
            "dyiurstuzhu": None,
            "tdovgfimofmclc": datetime.datetime(
                2023,
                1,
                15,
                15,
                23,
                38,
                686000,
                tzinfo=datetime.timezone.utc,
            ),
            "iyxkgbwxdlrdc": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                55,
                761000,
                tzinfo=datetime.timezone.utc,
            ),
            "jnjtckehsrtwhgzuhksmclk": ["tlejijcpbjzygepptbxgrugcbufncyupnivbljzhxe"],
            "zewoojzsiykjf": datetime.datetime(
                2023,
                1,
                17,
                11,
                17,
                46,
                140960,
                tzinfo=datetime.timezone.utc,
            ),
            "muzabbfnxptvqwzbeilkz": False,
            "wdiuepootdqyniogblxgwkgcqezutcesb": None,
            "lzkthufcerqnxdypdts": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                56,
                73000,
                tzinfo=datetime.timezone.utc,
            ),
            "epukgzafaubmn": 50000.0,
            "cdpeessdedncodoajdqsos": 50000.0,
            "adxucexfjgfwxo": "jwuoomwdrfklgt",
            "sotxdizdpuunbssidop": None,
            "lxmgvysiltbzfkjne": None,
            "wyeaarjbilfmjbfzjuzv": None,
            "cwlcgx": -1272.22,
            "oniptvyaub": -1275.75,
            "hqsfeelokxlwnha": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                55,
                886000,
                tzinfo=datetime.timezone.utc,
            ),
            "nuidlcyrxcrkyytgrnmc": -733.5,
            "wmofdeftonjcdnkg": -737.03,
            "bnsttxjfxxgxphfiguqew": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                55,
                886000,
                tzinfo=datetime.timezone.utc,
            ),
            "audhoqqxjliwnsqttwsadmwwv": -737.03,
            "badwwjzugwtdkbsamckoljfrrumtrt": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                55,
                886000,
                tzinfo=datetime.timezone.utc,
            ),
            "zlbggbbjgsugkgkqjycxwdx": -1241.28,
            "fxueeffryeafcxtkfzdmlmgu": -538.72,
            "yjmapfqummrsyujkosmixumjgfkwd": datetime.datetime(
                2023,
                1,
                16,
                22,
                59,
                59,
                999999,
                tzinfo=datetime.timezone.utc,
            ),
            "qepdxlodjetleseyminybdvitcgd": None,
            "ltokvpltajwbn": datetime.date(2023, 1, 17),
            "ifzhionnrpeoorsupiniwbljek": datetime.datetime(
                2023,
                1,
                17,
                11,
                19,
                49,
                113000,
                tzinfo=datetime.timezone.utc,
            ),
            "ljmmehacdawrlbhlhthm": -1241.28,
            "jnwffrtloedorwctsclshnpwjq": -702.56,
            "yhgssmtrmrcqhsdaekvoxyv": None,
            "nfzljididdzkofkrjfxdloygjxfhhoe": None,
            "mpctjlifbrgugaugiijj": None,
            "ckknohnsefzknbvnmwzlxlajsckl": None,
            "rfehqmeeslkcfbptrrghvivcrx": None,
            "nqeovbshknctkgkcytzbhfuvpcyamfrafi": None,
            "lptomdhvkvjnegsanzshqecas": 0,
            "vkbijuitbghlywkeojjf": None,
            "hzzmtggbqdglch": "xgehztikx",
            "yhmplqyhbndcfdafjvvr": False,
            "oucaxvjhjapayexuqwvnnls": None,
            "xbnagbhttfloffstxyr": 1673954411.5502248,
            "eiqrshvbjmlyzqle": {
                "dkayiglkkhfrvbliqy": ["ohjuifj"],
                "grqcjzqdiaslqaxhcqg": ["fuuxwsu"],
            },
            "uflenvgkk": {
                "ehycwsz": {
                    "jeikui": "noxawd",
                    "gkrefq": "hfonlfp",
                    "xkxs": "jzt",
                    "ztpmv": "mpscuot",
                    "zagmfzmgh": "pdculhh",
                    "jgzsrpukwqoln": 100000.0,
                    "vlqzkxbwc": datetime.datetime(
                        2023,
                        1,
                        17,
                        11,
                        19,
                        50,
                        867000,
                        tzinfo=datetime.timezone.utc,
                    ),
                    "cchovdmelbchcgvtg": -30.94,
                    "xvznnjfpwtdujqrh": 0.92059,
                    "tmsqwiiopyhlcovcxhojuzzyac": 1.0862009,
                    "tfzkaimjrpsbeswnrxeo": 0.0,
                    "isqjxmjupeiboufeaavkdj": -9.76,
                    "ywjqjiasfuifyqmz": 0.0,
                    "uvtlmdrk": 0.92028,
                    "dquzguej": None,
                    "guudreveynvhvhihegoybqrmejkj": datetime.datetime(
                        2023,
                        1,
                        17,
                        11,
                        19,
                        56,
                        73000,
                        tzinfo=datetime.timezone.utc,
                    ),
                    "agvnijfztpbpatej": "zym",
                    "mqsozcvnuvueixszfz": [
                        {
                            "oepzcayabl": "givcnhztbdmili",
                            "rhhaorqbiziqvyhglecqw": True,
                            "paxvrmateisxfqs": 1.0862009,
                            "bydrnmhvj": {
                                "kwqlickvqv": "beinfgmofalgytujorwxqfvlxtbeujmqwrdqzkfpul",
                                "cxdikf": "dfpbnpe",
                                "dnnhiy": "reeenz",
                                "tx": datetime.datetime(
                                    2023,
                                    1,
                                    17,
                                    11,
                                    19,
                                    56,
                                    73000,
                                    tzinfo=datetime.timezone.utc,
                                ),
                                "tck": datetime.date(2023, 1, 17),
                                "nvt": 0.92064,
                                "enc": 0.92059,
                                "icginezbybhcs": 1673954396073,
                                "gfamgxmknxirghgmtxl": 1673954411.5492423,
                            },
                        },
                    ],
                    "dqiabsyky": {
                        "hxzdtwunrr": "fozhshbmijhujcznqykxtlaxfbtdpzvwvjtyuqzlyw",
                        "tmpscl": "tbivvoa",
                        "vjjjvl": "arukeb",
                        "fm": datetime.datetime(
                            2023,
                            1,
                            17,
                            11,
                            19,
                            56,
                            73000,
                            tzinfo=datetime.timezone.utc,
                        ),
                        "rjq": datetime.date(2023, 1, 17),
                        "oax": 0.92064,
                        "gdv": 0.92059,
                        "vousomtllbpsh": 1673954396073,
                        "pgiblyqswxvwkpmpyay": 1673954411.5492423,
                    },
                    "gebil": [
                        {
                            "bzrjh": 0.92065,
                            "izmljcvqinm": 3.25,
                            "legczrbxlrmcep": None,
                        },
                    ],
                    "eqg": [
                        {
                            "yngp": "kako",
                            "udntq": {
                                "wzygahsmwd": "hplammnltegchpaorxaremhymtqtxdpfzzoyouimnw",
                                "iofcbwwgu": datetime.datetime(
                                    2023,
                                    1,
                                    17,
                                    11,
                                    19,
                                    50,
                                    867000,
                                    tzinfo=datetime.timezone.utc,
                                ),
                                "nengib": "zpyilz",
                                "sorpcw": "ixhzipg",
                                "kruw": "taq",
                                "vaqaj": "kravspj",
                                "omkjhzkxp": "watatag",
                                "ckwtjcqkjxmdn": 100000.0,
                                "kpjtgiuhfqx": 3.25,
                                "upkgqboyyg": 0.92065,
                                "gkshzyqtpmolnybr": 0.92065,
                                "oeiueaildnobcyzzpqwjwivkgj": 1.0861891,
                                "hiheqtjxyjnweryve": 0.0,
                                "wntcyohtaeylkylp": 0.0,
                                "jmebuufukzzymohzynpxzp": -9.76,
                                "rblubytyjuvbeurwrqmz": 0.0,
                                "xpscrgcnratymu": None,
                            },
                        },
                    ],
                    "kpmgmiqgmswzebawzciss": -0.7,
                    "ktggnjdemtfxhnultritqokgbjucktdiooic": 0.92058,
                    "oawdfonaymcwwvmszhdlemjcnb": datetime.datetime(
                        2023,
                        1,
                        17,
                        11,
                        19,
                        55,
                        886000,
                        tzinfo=datetime.timezone.utc,
                    ),
                    "bwfkzqjqqjdgbfbbjwoxhweihipy": "lzvn",
                    "feslxjpurrukajwellwjqww": 0.0,
                    "ptuysyuuhwkfqlugjlxkohwanzijtzknupfikp": None,
                    "gquuleqhpsbyiluhijdddreenggl": datetime.datetime(
                        2023,
                        1,
                        17,
                        11,
                        19,
                        50,
                        867000,
                        tzinfo=datetime.timezone.utc,
                    ),
                    "auhxrvhvvtszkkkpyhbhvpjlypjoyz": "vqdxfdvgxqcu",
                },
            },
        },
        "qbov": "vylhkevwf",
        "uidiyv": {
            "qkyoj": {
                "cclzxqbosqmj": 1673954395761,
                "rzijfrywwwcr": 1,
                "toujesmzk": "afnu",
                "aqmpunnlt": "nyreokscjljpfcrrstxgvwddphymgzkvuolbigqhla",
                "ofrjrk": "rlffwrw",
                "legyfjl": {
                    "byalenyqro": "tbzhyxo",
                    "qxrtujt": 0.92028,
                    "onmhbvy": 0,
                    "cbhmp": "vqrkzbg",
                },
            },
        },
    },
}


@needs_data
def test_issue331_1_pretty():
    as_bytes = read_fixture_bytes("issue331_1.json.xz")
    as_obj = orjson.loads(as_bytes)
    for _ in range(1000):
        assert orjson.loads(orjson.dumps(as_obj, option=orjson.OPT_INDENT_2)) == as_obj


@needs_data
def test_issue331_1_compact():
    as_bytes = read_fixture_bytes("issue331_1.json.xz")
    as_obj = orjson.loads(as_bytes)
    for _ in range(1000):
        assert orjson.loads(orjson.dumps(as_obj)) == as_obj


@needs_data
def test_issue331_2_pretty():
    as_bytes = read_fixture_bytes("issue331_2.json.xz")
    as_obj = orjson.loads(as_bytes)
    for _ in range(1000):
        assert orjson.loads(orjson.dumps(as_obj, option=orjson.OPT_INDENT_2)) == as_obj


@needs_data
def test_issue331_2_compact():
    as_bytes = read_fixture_bytes("issue331_2.json.xz")
    as_obj = orjson.loads(as_bytes)
    for _ in range(1000):
        assert orjson.loads(orjson.dumps(as_obj)) == as_obj


def test_issue335_compact():
    for _ in range(1000):
        assert orjson.dumps(FIXTURE_ISSUE_335)


def test_issue335_pretty():
    for _ in range(1000):
        assert orjson.dumps(FIXTURE_ISSUE_335, option=orjson.OPT_INDENT_2)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2018-2026)

import orjson

from .util import needs_data, read_fixture_str


@needs_data
class TestJsonChecker:
    def _run_roundtrip_json(self, filename, byte_exact=True):
        data = read_fixture_str(filename, "roundtrip")
        if byte_exact:
            assert orjson.dumps(orjson.loads(data)) == data.encode("utf-8")
        else:
            assert orjson.loads(orjson.dumps(orjson.loads(data))) == orjson.loads(data)

    def test_roundtrip001(self):
        """
        roundtrip001.json
        """
        self._run_roundtrip_json("roundtrip01.json")

    def test_roundtrip002(self):
        """
        roundtrip002.json
        """
        self._run_roundtrip_json("roundtrip02.json")

    def test_roundtrip003(self):
        """
        roundtrip003.json
        """
        self._run_roundtrip_json("roundtrip03.json")

    def test_roundtrip004(self):
        """
        roundtrip004.json
        """
        self._run_roundtrip_json("roundtrip04.json")

    def test_roundtrip005(self):
        """
        roundtrip005.json
        """
        self._run_roundtrip_json("roundtrip05.json")

    def test_roundtrip006(self):
        """
        roundtrip006.json
        """
        self._run_roundtrip_json("roundtrip06.json")

    def test_roundtrip007(self):
        """
        roundtrip007.json
        """
        self._run_roundtrip_json("roundtrip07.json")

    def test_roundtrip008(self):
        """
        roundtrip008.json
        """
        self._run_roundtrip_json("roundtrip08.json")

    def test_roundtrip009(self):
        """
        roundtrip009.json
        """
        self._run_roundtrip_json("roundtrip09.json")

    def test_roundtrip010(self):
        """
        roundtrip010.json
        """
        self._run_roundtrip_json("roundtrip10.json")

    def test_roundtrip011(self):
        """
        roundtrip011.json
        """
        self._run_roundtrip_json("roundtrip11.json")

    def test_roundtrip012(self):
        """
        roundtrip012.json
        """
        self._run_roundtrip_json("roundtrip12.json")

    def test_roundtrip013(self):
        """
        roundtrip013.json
        """
        self._run_roundtrip_json("roundtrip13.json")

    def test_roundtrip014(self):
        """
        roundtrip014.json
        """
        self._run_roundtrip_json("roundtrip14.json")

    def test_roundtrip015(self):
        """
        roundtrip015.json
        """
        self._run_roundtrip_json("roundtrip15.json")

    def test_roundtrip016(self):
        """
        roundtrip016.json
        """
        self._run_roundtrip_json("roundtrip16.json")

    def test_roundtrip017(self):
        """
        roundtrip017.json
        """
        self._run_roundtrip_json("roundtrip17.json")

    def test_roundtrip018(self):
        """
        roundtrip018.json
        """
        self._run_roundtrip_json("roundtrip18.json")

    def test_roundtrip019(self):
        """
        roundtrip019.json
        """
        self._run_roundtrip_json("roundtrip19.json")

    def test_roundtrip020(self):
        """
        roundtrip020.json
        """
        self._run_roundtrip_json("roundtrip20.json")

    def test_roundtrip021(self):
        """
        roundtrip021.json
        """
        self._run_roundtrip_json("roundtrip21.json")

    def test_roundtrip022(self):
        """
        roundtrip022.json
        """
        self._run_roundtrip_json("roundtrip22.json")

    def test_roundtrip023(self):
        """
        roundtrip023.json
        """
        self._run_roundtrip_json("roundtrip23.json")

    def test_roundtrip024(self):
        """
        roundtrip024.json
        """
        self._run_roundtrip_json("roundtrip24.json")

    def test_roundtrip025(self):
        """
        roundtrip025.json
        """
        self._run_roundtrip_json("roundtrip25.json")

    def test_roundtrip026(self):
        """
        roundtrip026.json
        """
        self._run_roundtrip_json("roundtrip26.json")

    def test_roundtrip027(self):
        """
        roundtrip027.json
        """
        self._run_roundtrip_json("roundtrip27.json", byte_exact=False)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)
# Copyright Eric Jolibois (2022), ijl (2023)

import orjson

orjson.JSONDecodeError(msg="the_msg", doc="the_doc", pos=1)

orjson.dumps(orjson.Fragment(b"{}"))



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2019-2023)

import pytest

import orjson


class TestCircular:
    def test_circular_dict(self):
        """
        dumps() circular reference dict
        """
        obj = {}  # type: ignore
        obj["obj"] = obj
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)

    def test_circular_dict_sort_keys(self):
        """
        dumps() circular reference dict OPT_SORT_KEYS
        """
        obj = {}  # type: ignore
        obj["obj"] = obj
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)

    def test_circular_dict_non_str_keys(self):
        """
        dumps() circular reference dict OPT_NON_STR_KEYS
        """
        obj = {}  # type: ignore
        obj["obj"] = obj
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_NON_STR_KEYS)

    def test_circular_list(self):
        """
        dumps() circular reference list
        """
        obj = []  # type: ignore
        obj.append(obj)  # type: ignore
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)

    def test_circular_nested(self):
        """
        dumps() circular reference nested dict, list
        """
        obj = {}  # type: ignore
        obj["list"] = [{"obj": obj}]
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj)

    def test_circular_nested_sort_keys(self):
        """
        dumps() circular reference nested dict, list OPT_SORT_KEYS
        """
        obj = {}  # type: ignore
        obj["list"] = [{"obj": obj}]
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)

    def test_circular_nested_non_str_keys(self):
        """
        dumps() circular reference nested dict, list OPT_NON_STR_KEYS
        """
        obj = {}  # type: ignore
        obj["list"] = [{"obj": obj}]
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_NON_STR_KEYS)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2019-2026)

import datetime

import pytest

import orjson

try:
    import zoneinfo

    _ = zoneinfo.ZoneInfo("Europe/Amsterdam")
except Exception:  # ImportError,ZoneInfoNotFoundError
    zoneinfo = None  # type: ignore

try:
    import pytz
except ImportError:
    pytz = None  # type: ignore

try:
    import pendulum
except ImportError:
    pendulum = None  # type: ignore

try:
    from dateutil import tz
except ImportError:
    tz = None  # type: ignore


AMSTERDAM_1937_DATETIMES = (
    b'["1937-01-01T12:00:27.000087+00:20"]',  # tzinfo<2022b and an example in RFC 3339
    b'["1937-01-01T12:00:27.000087+00:00"]',  # tzinfo>=2022b
)

AMSTERDAM_1937_DATETIMES_WITH_Z = (
    b'["1937-01-01T12:00:27.000087+00:20"]',
    b'["1937-01-01T12:00:27.000087Z"]',
)


class TestDatetime:
    def test_datetime_naive(self):
        """
        datetime.datetime naive prints without offset
        """
        assert (
            orjson.dumps([datetime.datetime(2000, 1, 1, 2, 3, 4, 123)])
            == b'["2000-01-01T02:03:04.000123"]'
        )

    def test_datetime_naive_utc(self):
        """
        datetime.datetime naive with opt assumes UTC
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["2000-01-01T02:03:04.000123+00:00"]'
        )

    def test_datetime_min(self):
        """
        datetime.datetime min range
        """
        assert (
            orjson.dumps(
                [datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0, 0, 0)],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["0001-01-01T00:00:00+00:00"]'
        )

    def test_datetime_min_invalid(self):
        """
        datetime.datetime min range invalid
        """
        with pytest.raises(ValueError):
            datetime.datetime(-1, 1, 1, 0, 0, 0, 0)

    def test_datetime_max_invalid(self):
        """
        datetime.datetime max range invalid
        """
        with pytest.raises(ValueError):
            datetime.datetime(10000, 1, 1, 0, 0, 0, 0)

    def test_datetime_max(self):
        """
        datetime.datetime max range
        """
        assert (
            orjson.dumps(
                [datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 50, 999999)],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["9999-12-31T23:59:50.999999+00:00"]'
        )

    def test_datetime_three_digits(self):
        """
        datetime.datetime three digit year
        """
        assert (
            orjson.dumps(
                [datetime.datetime(312, 1, 1)],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["0312-01-01T00:00:00+00:00"]'
        )

    def test_datetime_two_digits(self):
        """
        datetime.datetime two digit year
        """
        assert (
            orjson.dumps(
                [datetime.datetime(46, 1, 1)],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["0046-01-01T00:00:00+00:00"]'
        )

    @pytest.mark.skipif(tz is None, reason="dateutil optional")
    def test_datetime_tz_assume(self):
        """
        datetime.datetime tz with assume UTC uses tz
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        1,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=tz.gettz("Asia/Shanghai"),
                    ),
                ],
                option=orjson.OPT_NAIVE_UTC,
            )
            == b'["2018-01-01T02:03:04+08:00"]'
        )

    def test_datetime_timezone_utc(self):
        """
        datetime.datetime.utc
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        6,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=datetime.timezone.utc,
                    ),
                ],
            )
            == b'["2018-06-01T02:03:04+00:00"]'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_pytz_utc(self):
        """
        pytz.UTC
        """
        assert (
            orjson.dumps([datetime.datetime(2018, 6, 1, 2, 3, 4, 0, tzinfo=pytz.UTC)])
            == b'["2018-06-01T02:03:04+00:00"]'
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_zoneinfo_utc(self):
        """
        zoneinfo.ZoneInfo("UTC")
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        6,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=zoneinfo.ZoneInfo("UTC"),
                    ),
                ],
            )
            == b'["2018-06-01T02:03:04+00:00"]'
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_zoneinfo_positive(self):
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        1,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=zoneinfo.ZoneInfo("Asia/Shanghai"),
                    ),
                ],
            )
            == b'["2018-01-01T02:03:04+08:00"]'
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_zoneinfo_negative(self):
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        6,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=zoneinfo.ZoneInfo("America/New_York"),
                    ),
                ],
            )
            == b'["2018-06-01T02:03:04-04:00"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_pendulum_utc(self):
        """
        datetime.datetime UTC
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2018, 6, 1, 2, 3, 4, 0, tzinfo=pendulum.UTC)],
            )
            == b'["2018-06-01T02:03:04+00:00"]'
        )

    @pytest.mark.skipif(tz is None, reason="dateutil optional")
    def test_datetime_arrow_positive(self):
        """
        datetime.datetime positive UTC
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        1,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=tz.gettz("Asia/Shanghai"),
                    ),
                ],
            )
            == b'["2018-01-01T02:03:04+08:00"]'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_pytz_positive(self):
        """
        datetime.datetime positive UTC
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        1,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pytz.timezone("Asia/Shanghai"),
                    ),
                ],
            )
            == b'["2018-01-01T02:03:04+08:00"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_pendulum_positive(self):
        """
        datetime.datetime positive UTC
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        1,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pendulum.timezone("Asia/Shanghai"),  # type: ignore
                    ),
                ],
            )
            == b'["2018-01-01T02:03:04+08:00"]'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_pytz_negative_dst(self):
        """
        datetime.datetime negative UTC DST
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        6,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pytz.timezone("America/New_York"),
                    ),
                ],
            )
            == b'["2018-06-01T02:03:04-04:00"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_pendulum_negative_dst(self):
        """
        datetime.datetime negative UTC DST
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        6,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pendulum.timezone("America/New_York"),  # type: ignore
                    ),
                ],
            )
            == b'["2018-06-01T02:03:04-04:00"]'
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_zoneinfo_negative_non_dst(self):
        """
        datetime.datetime negative UTC non-DST
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=zoneinfo.ZoneInfo("America/New_York"),
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04-05:00"]'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_pytz_negative_non_dst(self):
        """
        datetime.datetime negative UTC non-DST
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pytz.timezone("America/New_York"),
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04-05:00"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_pendulum_negative_non_dst(self):
        """
        datetime.datetime negative UTC non-DST
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pendulum.timezone("America/New_York"),  # type: ignore
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04-05:00"]'
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_zoneinfo_partial_hour(self):
        """
        datetime.datetime UTC offset partial hour
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=zoneinfo.ZoneInfo("Australia/Adelaide"),
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04+10:30"]'
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_pytz_partial_hour(self):
        """
        datetime.datetime UTC offset partial hour
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pytz.timezone("Australia/Adelaide"),
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04+10:30"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_pendulum_partial_hour(self):
        """
        datetime.datetime UTC offset partial hour
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2018,
                        12,
                        1,
                        2,
                        3,
                        4,
                        0,
                        tzinfo=pendulum.timezone("Australia/Adelaide"),  # type: ignore
                    ),
                ],
            )
            == b'["2018-12-01T02:03:04+10:30"]'
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_partial_second_pendulum_supported(self):
        """
        datetime.datetime UTC offset round seconds

        https://tools.ietf.org/html/rfc3339#section-5.8
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        1937,
                        1,
                        1,
                        12,
                        0,
                        27,
                        87,
                        tzinfo=pendulum.timezone("Europe/Amsterdam"),  # type: ignore
                    ),
                ],
            )
            in AMSTERDAM_1937_DATETIMES
        )

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_datetime_partial_second_zoneinfo(self):
        """
        datetime.datetime UTC offset round seconds

        https://tools.ietf.org/html/rfc3339#section-5.8
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        1937,
                        1,
                        1,
                        12,
                        0,
                        27,
                        87,
                        tzinfo=zoneinfo.ZoneInfo("Europe/Amsterdam"),
                    ),
                ],
            )
            in AMSTERDAM_1937_DATETIMES
        )

    @pytest.mark.skipif(pytz is None, reason="pytz optional")
    def test_datetime_partial_second_pytz(self):
        """
        datetime.datetime UTC offset round seconds

        https://tools.ietf.org/html/rfc3339#section-5.8
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        1937,
                        1,
                        1,
                        12,
                        0,
                        27,
                        87,
                        tzinfo=pytz.timezone("Europe/Amsterdam"),
                    ),
                ],
            )
            in AMSTERDAM_1937_DATETIMES
        )

    @pytest.mark.skipif(tz is None, reason="dateutil optional")
    def test_datetime_partial_second_dateutil(self):
        """
        datetime.datetime UTC offset round seconds

        https://tools.ietf.org/html/rfc3339#section-5.8
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        1937,
                        1,
                        1,
                        12,
                        0,
                        27,
                        87,
                        tzinfo=tz.gettz("Europe/Amsterdam"),
                    ),
                ],
            )
            in AMSTERDAM_1937_DATETIMES
        )

    def test_datetime_microsecond_max(self):
        """
        datetime.datetime microsecond max
        """
        assert (
            orjson.dumps(datetime.datetime(2000, 1, 1, 0, 0, 0, 999999))
            == b'"2000-01-01T00:00:00.999999"'
        )

    def test_datetime_microsecond_min(self):
        """
        datetime.datetime microsecond min
        """
        assert (
            orjson.dumps(datetime.datetime(2000, 1, 1, 0, 0, 0, 1))
            == b'"2000-01-01T00:00:00.000001"'
        )

    def test_datetime_omit_microseconds(self):
        """
        datetime.datetime OPT_OMIT_MICROSECONDS
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_OMIT_MICROSECONDS,
            )
            == b'["2000-01-01T02:03:04"]'
        )

    def test_datetime_omit_microseconds_naive(self):
        """
        datetime.datetime naive OPT_OMIT_MICROSECONDS
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_NAIVE_UTC | orjson.OPT_OMIT_MICROSECONDS,
            )
            == b'["2000-01-01T02:03:04+00:00"]'
        )

    def test_time_omit_microseconds(self):
        """
        datetime.time OPT_OMIT_MICROSECONDS
        """
        assert (
            orjson.dumps(
                [datetime.time(2, 3, 4, 123)],
                option=orjson.OPT_OMIT_MICROSECONDS,
            )
            == b'["02:03:04"]'
        )

    def test_datetime_utc_z_naive_omit(self):
        """
        datetime.datetime naive OPT_UTC_Z
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_NAIVE_UTC
                | orjson.OPT_UTC_Z
                | orjson.OPT_OMIT_MICROSECONDS,
            )
            == b'["2000-01-01T02:03:04Z"]'
        )

    def test_datetime_utc_z_naive(self):
        """
        datetime.datetime naive OPT_UTC_Z
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_NAIVE_UTC | orjson.OPT_UTC_Z,
            )
            == b'["2000-01-01T02:03:04.000123Z"]'
        )

    def test_datetime_utc_z_without_tz(self):
        """
        datetime.datetime naive OPT_UTC_Z
        """
        assert (
            orjson.dumps(
                [datetime.datetime(2000, 1, 1, 2, 3, 4, 123)],
                option=orjson.OPT_UTC_Z,
            )
            == b'["2000-01-01T02:03:04.000123"]'
        )

    @pytest.mark.skipif(tz is None, reason="dateutil optional")
    def test_datetime_utc_z_with_tz(self):
        """
        datetime.datetime naive OPT_UTC_Z
        """
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        2000,
                        1,
                        1,
                        0,
                        0,
                        0,
                        1,
                        tzinfo=datetime.timezone.utc,
                    ),
                ],
                option=orjson.OPT_UTC_Z,
            )
            == b'["2000-01-01T00:00:00.000001Z"]'
        )
        assert (
            orjson.dumps(
                [
                    datetime.datetime(
                        1937,
                        1,
                        1,
                        12,
                        0,
                        27,
                        87,
                        tzinfo=tz.gettz("Europe/Amsterdam"),
                    ),
                ],
                option=orjson.OPT_UTC_Z,
            )
            in AMSTERDAM_1937_DATETIMES_WITH_Z
        )

    @pytest.mark.skipif(pendulum is None, reason="pendulum not installed")
    def test_datetime_roundtrip(self):
        """
        datetime.datetime parsed by pendulum
        """
        obj = datetime.datetime(2000, 1, 1, 0, 0, 0, 1, tzinfo=datetime.timezone.utc)
        serialized = orjson.dumps(obj).decode("utf-8").replace('"', "")
        parsed = pendulum.parse(serialized)
        for attr in ("year", "month", "day", "hour", "minute", "second", "microsecond"):
            assert getattr(obj, attr) == getattr(parsed, attr)


class TestDate:
    def test_date(self):
        """
        datetime.date
        """
        assert orjson.dumps([datetime.date(2000, 1, 13)]) == b'["2000-01-13"]'

    def test_date_min(self):
        """
        datetime.date MINYEAR
        """
        assert (
            orjson.dumps([datetime.date(datetime.MINYEAR, 1, 1)]) == b'["0001-01-01"]'
        )

    def test_date_max(self):
        """
        datetime.date MAXYEAR
        """
        assert (
            orjson.dumps([datetime.date(datetime.MAXYEAR, 12, 31)]) == b'["9999-12-31"]'
        )

    def test_date_three_digits(self):
        """
        datetime.date three digit year
        """
        assert (
            orjson.dumps(
                [datetime.date(312, 1, 1)],
            )
            == b'["0312-01-01"]'
        )

    def test_date_two_digits(self):
        """
        datetime.date two digit year
        """
        assert (
            orjson.dumps(
                [datetime.date(46, 1, 1)],
            )
            == b'["0046-01-01"]'
        )


class TestTime:
    def test_time(self):
        """
        datetime.time
        """
        assert orjson.dumps([datetime.time(12, 15, 59, 111)]) == b'["12:15:59.000111"]'
        assert orjson.dumps([datetime.time(12, 15, 59)]) == b'["12:15:59"]'

    @pytest.mark.skipif(zoneinfo is None, reason="zoneinfo not available")
    def test_time_tz(self):
        """
        datetime.time with tzinfo error
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(
                [
                    datetime.time(
                        12,
                        15,
                        59,
                        111,
                        tzinfo=zoneinfo.ZoneInfo("Asia/Shanghai"),
                    ),
                ],
            )

    def test_time_microsecond_max(self):
        """
        datetime.time microsecond max
        """
        assert orjson.dumps(datetime.time(0, 0, 0, 999999)) == b'"00:00:00.999999"'

    def test_time_microsecond_min(self):
        """
        datetime.time microsecond min
        """
        assert orjson.dumps(datetime.time(0, 0, 0, 1)) == b'"00:00:00.000001"'


class TestDateclassPassthrough:
    def test_passthrough_datetime(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(
                datetime.datetime(1970, 1, 1),
                option=orjson.OPT_PASSTHROUGH_DATETIME,
            )

    def test_passthrough_date(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(
                datetime.date(1970, 1, 1),
                option=orjson.OPT_PASSTHROUGH_DATETIME,
            )

    def test_passthrough_time(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(
                datetime.time(12, 0, 0),
                option=orjson.OPT_PASSTHROUGH_DATETIME,
            )

    def test_passthrough_datetime_default(self):
        def default(obj):
            return obj.strftime("%a, %d %b %Y %H:%M:%S GMT")

        assert (
            orjson.dumps(
                datetime.datetime(1970, 1, 1),
                option=orjson.OPT_PASSTHROUGH_DATETIME,
                default=default,
            )
            == b'"Thu, 01 Jan 1970 00:00:00 GMT"'
        )



# SPDX-License-Identifier: (Apache-2.0 OR MIT)
# Copyright ijl (2020-2026), Aarni Koskela (2021)

from json import loads as json_loads

import pytest

from .data import FIXTURE_AS_OBJECTS, FIXTURE_NAMES, LIBRARIES


@pytest.mark.parametrize("library", LIBRARIES)
@pytest.mark.parametrize("fixture", FIXTURE_NAMES)
def test_dumps(benchmark, fixture, library):
    dumper, _ = LIBRARIES[library]
    benchmark.group = f"{fixture} serialization"
    benchmark.extra_info["lib"] = library
    data = FIXTURE_AS_OBJECTS[fixture]
    benchmark.extra_info["correct"] = json_loads(dumper(data)) == data  # type: ignore
    benchmark(dumper, data)



# SPDX-License-Identifier: MPL-2.0
# Copyright ijl (2019-2026)

import abc
import uuid
from dataclasses import InitVar, asdict, dataclass, field
from enum import Enum
from typing import ClassVar, Optional

import pytest

import orjson


class AnEnum(Enum):
    ONE = 1
    TWO = 2


@dataclass
class EmptyDataclass:
    pass


@dataclass
class EmptyDataclassSlots:
    __slots__ = ()


@dataclass
class Dataclass1:
    name: str
    number: int
    sub: Optional["Dataclass1"]


@dataclass
class Dataclass2:
    name: str | None = field(default="?")


@dataclass
class Dataclass3:
    a: str
    b: int
    c: dict
    d: bool
    e: float
    f: list
    g: tuple


@dataclass
class Dataclass4:
    a: str = field()
    b: int = field(metadata={"unrelated": False})
    c: float = 1.1


@dataclass
class Datasubclass(Dataclass1):
    additional: bool


@dataclass
class Slotsdataclass:
    __slots__ = ("_c", "a", "b", "d")
    a: str
    b: int
    _c: str
    d: InitVar[str]
    cls_var: ClassVar[str] = "cls"


@dataclass
class Defaultdataclass:
    a: uuid.UUID
    b: AnEnum


@dataclass
class UnsortedDataclass:
    c: int
    b: int
    a: int
    d: dict | None


@dataclass
class InitDataclass:
    a: InitVar[str]
    b: InitVar[str]
    cls_var: ClassVar[str] = "cls"
    ab: str = ""

    def __post_init__(self, a: str, b: str):
        self._other = 1
        self.ab = f"{a} {b}"


class AbstractBase(abc.ABC):
    @abc.abstractmethod
    def key(self):
        raise NotImplementedError


@dataclass(frozen=True)
class ConcreteAbc(AbstractBase):
    __slots__ = ("attr",)

    attr: float

    def key(self):
        return "dkjf"


class TestDataclass:
    def test_dataclass(self):
        """
        dumps() dataclass
        """
        obj = Dataclass1("a", 1, None)
        assert orjson.dumps(obj) == b'{"name":"a","number":1,"sub":null}'

    def test_dataclass_recursive(self):
        """
        dumps() dataclass recursive
        """
        obj = Dataclass1("a", 1, Dataclass1("b", 2, None))
        assert (
            orjson.dumps(obj)
            == b'{"name":"a","number":1,"sub":{"name":"b","number":2,"sub":null}}'
        )

    def test_dataclass_circular(self):
        """
        dumps() dataclass circular
        """
        obj1 = Dataclass1("a", 1, None)
        obj2 = Dataclass1("b", 2, obj1)
        obj1.sub = obj2
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj1)

    def test_dataclass_empty(self):
        """
        dumps() no attributes
        """
        assert orjson.dumps(EmptyDataclass()) == b"{}"

    def test_dataclass_empty_slots(self):
        """
        dumps() no attributes slots
        """
        assert orjson.dumps(EmptyDataclassSlots()) == b"{}"

    def test_dataclass_default_arg(self):
        """
        dumps() dataclass default arg
        """
        obj = Dataclass2()
        assert orjson.dumps(obj) == b'{"name":"?"}'

    def test_dataclass_types(self):
        """
        dumps() dataclass types
        """
        obj = Dataclass3("a", 1, {"a": "b"}, True, 1.1, [1, 2], (3, 4))
        assert (
            orjson.dumps(obj)
            == b'{"a":"a","b":1,"c":{"a":"b"},"d":true,"e":1.1,"f":[1,2],"g":[3,4]}'
        )

    def test_dataclass_metadata(self):
        """
        dumps() dataclass metadata
        """
        obj = Dataclass4("a", 1, 2.1)
        assert orjson.dumps(obj) == b'{"a":"a","b":1,"c":2.1}'

    def test_dataclass_classvar(self):
        """
        dumps() dataclass class variable
        """
        obj = Dataclass4("a", 1)
        assert orjson.dumps(obj) == b'{"a":"a","b":1,"c":1.1}'

    def test_dataclass_subclass(self):
        """
        dumps() dataclass subclass
        """
        obj = Datasubclass("a", 1, None, False)
        assert (
            orjson.dumps(obj)
            == b'{"name":"a","number":1,"sub":null,"additional":false}'
        )

    def test_dataclass_slots(self):
        """
        dumps() dataclass with __slots__ does not include under attributes, InitVar, or ClassVar
        """
        obj = Slotsdataclass("a", 1, "c", "d")
        assert "__dict__" not in dir(obj)
        assert orjson.dumps(obj) == b'{"a":"a","b":1}'

    def test_dataclass_default(self):
        """
        dumps() dataclass with default
        """

        def default(__obj):
            if isinstance(__obj, uuid.UUID):
                return str(__obj)
            elif isinstance(__obj, Enum):
                return __obj.value

        obj = Defaultdataclass(
            uuid.UUID("808989c0-00d5-48a8-b5c4-c804bf9032f2"),
            AnEnum.ONE,
        )
        assert (
            orjson.dumps(obj, default=default)
            == b'{"a":"808989c0-00d5-48a8-b5c4-c804bf9032f2","b":1}'
        )

    def test_dataclass_sort(self):
        """
        OPT_SORT_KEYS has no effect on dataclasses
        """
        obj = UnsortedDataclass(1, 2, 3, None)
        assert (
            orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
            == b'{"c":1,"b":2,"a":3,"d":null}'
        )

    def test_dataclass_sort_sub(self):
        """
        dataclass fast path does not prevent OPT_SORT_KEYS from cascading
        """
        obj = UnsortedDataclass(1, 2, 3, {"f": 2, "e": 1})
        assert (
            orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
            == b'{"c":1,"b":2,"a":3,"d":{"e":1,"f":2}}'
        )

    def test_dataclass_under(self):
        """
        dumps() does not include under attributes, InitVar, or ClassVar
        """
        obj = InitDataclass("zxc", "vbn")
        assert orjson.dumps(obj) == b'{"ab":"zxc vbn"}'

    def test_dataclass_option(self):
        """
        dumps() accepts deprecated OPT_SERIALIZE_DATACLASS
        """
        obj = Dataclass1("a", 1, None)
        assert (
            orjson.dumps(obj, option=orjson.OPT_SERIALIZE_DATACLASS)
            == b'{"name":"a","number":1,"sub":null}'
        )


class TestDataclassPassthrough:
    def test_dataclass_passthrough_raise(self):
        """
        dumps() dataclass passes to default with OPT_PASSTHROUGH_DATACLASS
        """
        obj = Dataclass1("a", 1, None)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(obj, option=orjson.OPT_PASSTHROUGH_DATACLASS)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(
                InitDataclass("zxc", "vbn"),
                option=orjson.OPT_PASSTHROUGH_DATACLASS,
            )

    def test_dataclass_passthrough_default(self):
        """
        dumps() dataclass passes to default with OPT_PASSTHROUGH_DATACLASS
        """
        obj = Dataclass1("a", 1, None)
        assert (
            orjson.dumps(obj, option=orjson.OPT_PASSTHROUGH_DATACLASS, default=asdict)
            == b'{"name":"a","number":1,"sub":null}'
        )

        def default(obj):
            if isinstance(obj, Dataclass1):
                return {"name": obj.name, "number": obj.number}
            raise TypeError

        assert (
            orjson.dumps(obj, option=orjson.OPT_PASSTHROUGH_DATACLASS, default=default)
            == b'{"name":"a","number":1}'
        )


class TestAbstractDataclass:
    def test_dataclass_abc(self):
        obj = ConcreteAbc(1.0)
        assert orjson.dumps(obj) == b'{"attr":1.0}'
